@@ -4,7 +4,7 @@
 #
 # Copyright © 1996 Ian Jackson
 # Copyright © 2000 Wichert Akkerman
-# Copyright © 2006-2010,2012-2013 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2010,2012-2014 Guillem Jover <guillem@debian.org>
 # Copyright © 2007 Frank Lichtenheld
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 use strict;
 use warnings;
 
+use Carp;
 use Cwd;
 use File::Basename;
 use POSIX qw(:sys_wait_h);
@@ -33,7 +34,9 @@ use Dpkg::ErrorHandling;
 use Dpkg::BuildOptions;
 use Dpkg::BuildProfiles qw(set_build_profiles);
 use Dpkg::Compression;
+use Dpkg::Checksums;
 use Dpkg::Version;
+use Dpkg::Control;
 use Dpkg::Changelog::Parse;
 use Dpkg::Path qw(find_command);
 use Dpkg::IPC;
@@ -55,10 +58,12 @@ sub usage {
     . "\n\n" . _g(
 'Options:
   -F (default)   normal full build (binaries and sources).
-  -b             binary-only, do not build source.
-  -B             binary-only, no arch-indep files.
+  -g             source and arch-indep build.
+  -G             source and arch-specific build.
+  -b             binary-only, no source files.
+  -B             binary-only, only arch-specific files.
   -A             binary-only, only arch-indep files.
-  -S             source only, no binary files.
+  -S             source-only, no binary files.
   -nc            do not clean source tree (implies -b).
   -tc            clean source tree when finished.
   -D (default)   check build dependencies and conflicts.
@@ -70,6 +75,14 @@ sub usage {
   -j[<number>]   specify jobs to run simultaneously (passed to <rules>).
   -r<gain-root-command>
                  command to gain root privileges (default is fakeroot).
+  --check-command=<check-command>
+                 command to check the .changes file (no default).
+  --check-option=<opt>
+                 pass <opt> to <check-command>.
+  --hook-<hook-name>=<hook-command>
+                 set <hook-command> as the hook <hook-name>, known hooks:
+                   init preclean source build binary changes postclean
+                   check sign done
   -p<sign-command>
                  command to sign .dsc and/or .changes files
                    (default is gpg2 or gpg).
@@ -89,9 +102,9 @@ sub usage {
   -t<system>     set GNU system type.')
     . "\n\n" . _g(
 'Options passed to dpkg-genchanges:
-  -si (default)  source includes orig if new upstream.
-  -sa            uploaded source always includes orig.
-  -sd            uploaded source is diff and .dsc only.
+  -si (default)  source includes orig, if new upstream.
+  -sa            source includes orig, always.
+  -sd            source is diff and .dsc only.
   -v<version>    changes since version <version>.
   -m<maint>      maintainer for package is <maint>.
   -e<maint>      maintainer for release is <maint>.
@@ -111,50 +124,94 @@ sub usage {
 '), $Dpkg::PROGNAME;
 }
 
+my $admindir;
 my @debian_rules = ('debian/rules');
 my @rootcommand = ();
 my $signcommand;
-my ($admindir, $usepause, $noclean,
-    $cleansource, $since, $maint,
-    $changedby, $desc, $parallel);
+my $noclean;
+my $cleansource;
+my $parallel;
 my $checkbuilddep = 1;
-my $signkey = defined $ENV{DEB_SIGN_KEYID} ? $ENV{DEB_SIGN_KEYID} : undef;
+my @source_opts;
+my $check_command = $ENV{DEB_CHECK_COMMAND};
+my @check_opts;
+my $signpause;
+my $signkey = $ENV{DEB_SIGN_KEYID};
 my $signforce = 0;
 my $signreleased = 1;
 my $signsource = 1;
 my $signchanges = 1;
 my $buildtarget = 'build';
 my $binarytarget = 'binary';
-my $targetarch = my $targetgnusystem = '';
+my $targetarch = '';
+my $targetgnusystem = '';
 my @build_profiles = ();
 my $call_target = '';
 my $call_target_as_root = 0;
-my (@checkbuilddep_opts, @changes_opts, @source_opts);
+my $since;
+my $maint;
+my $changedby;
+my $desc;
+my @changes_opts;
+my @hook_names = qw(
+    init preclean source build binary changes postclean check sign done
+);
+my %hook;
+$hook{$_} = undef foreach @hook_names;
 
 use constant BUILD_DEFAULT    => 1;
 use constant BUILD_SOURCE     => 2;
 use constant BUILD_ARCH_DEP   => 4;
 use constant BUILD_ARCH_INDEP => 8;
 use constant BUILD_BINARY     => BUILD_ARCH_DEP | BUILD_ARCH_INDEP;
+use constant BUILD_SOURCE_DEP => BUILD_SOURCE | BUILD_ARCH_DEP;
+use constant BUILD_SOURCE_INDEP => BUILD_SOURCE | BUILD_ARCH_INDEP;
 use constant BUILD_ALL        => BUILD_BINARY | BUILD_SOURCE;
 my $include = BUILD_ALL | BUILD_DEFAULT;
 
-sub build_normal() { return ($include & BUILD_ALL) == BUILD_ALL; }
+sub build_is_default() { return $include & BUILD_DEFAULT; }
 sub build_sourceonly() { return $include == BUILD_SOURCE; }
 sub build_binaryonly() { return !($include & BUILD_SOURCE); }
 sub build_binaryindep() { return ($include == BUILD_ARCH_INDEP); }
-sub build_opt() {
-    return (($include == BUILD_BINARY) ? '-b' :
-            (($include == BUILD_ARCH_DEP) ? '-B' :
-             (($include == BUILD_ARCH_INDEP) ? '-A' :
-              (($include == BUILD_SOURCE) ? '-S' :
-               internerr("build_opt called with include=$include")))));
+sub build_opt {
+    if ($include == BUILD_BINARY) {
+        return '-b';
+    } elsif ($include == BUILD_ARCH_DEP) {
+        return '-B';
+    } elsif ($include == BUILD_ARCH_INDEP) {
+        return '-A';
+    } elsif ($include == BUILD_SOURCE) {
+        return '-S';
+    } elsif ($include == BUILD_SOURCE_DEP) {
+        return '-G';
+    } elsif ($include == BUILD_SOURCE_INDEP) {
+        return '-g';
+    } else {
+        croak "build_opt called with include=$include";
+    }
+}
+
+sub set_build_type
+{
+    my ($build_type, $build_option) = @_;
+
+    usageerr(_g('cannot combine %s and %s'), build_opt(), $build_option)
+        if not build_is_default and $include != $build_type;
+    $include = $build_type;
+}
+
+my $build_opts = Dpkg::BuildOptions->new();
+
+if ($build_opts->has('nocheck')) {
+    $check_command = undef;
+} elsif (not find_command($check_command)) {
+    $check_command = undef;
 }
 
 while (@ARGV) {
     $_ = shift @ARGV;
 
-    if (/^(--help|-\?)$/) {
+    if (/^(?:--help|-\?)$/) {
 	usage;
 	exit 0;
     } elsif (/^--version$/) {
@@ -168,10 +225,22 @@ while (@ARGV) {
 	push @source_opts, $1;
     } elsif (/^--changes-option=(.*)$/) {
 	push @changes_opts, $1;
-    } elsif (/^-j(\d*)$/) {
+    } elsif (/^-j(\d*|auto)$/) {
 	$parallel = $1 || '';
     } elsif (/^-r(.*)$/) {
-	@rootcommand = split /\s+/, $1;
+	my $arg = $1;
+	@rootcommand = split /\s+/, $arg;
+    } elsif (/^--check-command=(.*)$/) {
+	$check_command = $1;
+    } elsif (/^--check-option=(.*)$/) {
+	push @check_opts, $1;
+    } elsif (/^--hook-(.+)=(.*)$/) {
+	my ($hook_name, $hook_cmd) = ($1, $2);
+	usageerr(_g('unknown hook name %s'), $hook_name)
+	    if not exists $hook{$hook_name};
+	usageerr(_g('missing hook %s command'), $hook_name)
+	    if not defined $hook_cmd;
+	$hook{$hook_name} = $hook_cmd;
     } elsif (/^-p(.*)$/) {
 	$signcommand = $1;
     } elsif (/^-k(.*)$/) {
@@ -188,11 +257,12 @@ while (@ARGV) {
     } elsif (/^-uc$/) {
 	$signchanges = 0;
     } elsif (/^-ap$/) {
-	$usepause = 1;
+	$signpause = 1;
     } elsif (/^-a(.*)$/) {
 	$targetarch = $1;
     } elsif (/^-P(.*)$/) {
-	@build_profiles = split /,/, $1;
+	my $arg = $1;
+	@build_profiles = split /,/, $arg;
     } elsif (/^-s[iad]$/) {
 	push @changes_opts, $_;
     } elsif (/^-(?:s[insAkurKUR]|[zZ].*|i.*|I.*)$/) {
@@ -201,46 +271,34 @@ while (@ARGV) {
 	$cleansource = 1;
     } elsif (/^-t(.*)$/) {
 	$targetgnusystem = $1; # Order DOES matter!
-    } elsif (/^(--target|-T)$/) {
+    } elsif (/^(?:--target|-T)$/) {
         $call_target = shift @ARGV;
-    } elsif (/^(--target=|-T)(.+)$/) {
-        $call_target = $2;
+    } elsif (/^(?:--target=|-T)(.+)$/) {
+        $call_target = $1;
     } elsif (/^--as-root$/) {
         $call_target_as_root = 1;
     } elsif (/^-nc$/) {
 	$noclean = 1;
     } elsif (/^-b$/) {
-	usageerr(_g('cannot combine %s and %s'), $_, '-S') if build_sourceonly;
-	$include = BUILD_BINARY;
+	set_build_type(BUILD_BINARY, $_);
 	push @changes_opts, '-b';
-	@checkbuilddep_opts = ();
-	$buildtarget = 'build';
-	$binarytarget = 'binary';
     } elsif (/^-B$/) {
-	usageerr(_g('cannot combine %s and %s'), $_, '-S') if build_sourceonly;
-	$include = BUILD_ARCH_DEP;
+	set_build_type(BUILD_ARCH_DEP, $_);
 	push @changes_opts, '-B';
-	@checkbuilddep_opts = qw(-B);
-	$buildtarget = 'build-arch';
-	$binarytarget = 'binary-arch';
     } elsif (/^-A$/) {
-	usageerr(_g('cannot combine %s and %s'), $_, '-S') if build_sourceonly;
-	$include = BUILD_ARCH_INDEP;
+	set_build_type(BUILD_ARCH_INDEP, $_);
 	push @changes_opts, '-A';
-	@checkbuilddep_opts = qw(-A);
-	$buildtarget = 'build-indep';
-	$binarytarget = 'binary-indep';
     } elsif (/^-S$/) {
-	usageerr(_g('cannot combine %s and %s'), build_opt, '-S')
-	    if build_binaryonly;
-	$include = BUILD_SOURCE;
+	set_build_type(BUILD_SOURCE, $_);
 	push @changes_opts, '-S';
-	@checkbuilddep_opts = qw(-A -B);
+    } elsif (/^-G$/) {
+	set_build_type(BUILD_SOURCE_DEP, $_);
+	push @changes_opts, '-G';
+    } elsif (/^-g$/) {
+	set_build_type(BUILD_SOURCE_INDEP, $_);
+	push @changes_opts, '-g';
     } elsif (/^-F$/) {
-	usageerr(_g('cannot combine %s and %s'), $_, build_opt)
-	    if not build_normal;
-	$include = BUILD_ALL;
-	@checkbuilddep_opts = ();
+	set_build_type(BUILD_ALL, $_);
     } elsif (/^-v(.*)$/) {
 	$since = $1;
     } elsif (/^-m(.*)$/) {
@@ -253,37 +311,72 @@ while (@ARGV) {
 	# Deprecated option
 	warning(_g('-E and -W are deprecated, they are without effect'));
     } elsif (/^-R(.*)$/) {
-	@debian_rules = split /\s+/, $1;
+	my $arg = $1;
+	@debian_rules = split /\s+/, $arg;
     } else {
 	usageerr(_g('unknown option or argument %s'), $_);
     }
 }
 
+if (($include & BUILD_BINARY) == BUILD_BINARY) {
+    $buildtarget = 'build';
+    $binarytarget = 'binary';
+} elsif ($include & BUILD_ARCH_DEP) {
+    $buildtarget = 'build-arch';
+    $binarytarget = 'binary-arch';
+} elsif ($include & BUILD_ARCH_INDEP) {
+    $buildtarget = 'build-indep';
+    $binarytarget = 'binary-indep';
+}
+
 if ($noclean) {
     # -nc without -b/-B/-A/-S/-F implies -b
-    $include = BUILD_BINARY if ($include & BUILD_DEFAULT);
+    $include = BUILD_BINARY if build_is_default;
 }
 
 if ($< == 0) {
     warning(_g('using a gain-root-command while being root')) if (@rootcommand);
 } else {
     push @rootcommand, 'fakeroot' unless @rootcommand;
+}
 
-    if (!find_command($rootcommand[0])) {
-	if ($rootcommand[0] eq 'fakeroot') {
-	    error(_g("fakeroot not found, either install the fakeroot\n" .
-	             'package, specify a command with the -r option, ' .
-	             'or run this as root'));
-	} else {
-	    error(_g("gain-root-commmand '%s' not found"), $rootcommand[0]);
-	}
+if (@rootcommand and not find_command($rootcommand[0])) {
+    if ($rootcommand[0] eq 'fakeroot' and $< != 0) {
+        error(_g("fakeroot not found, either install the fakeroot\n" .
+                 'package, specify a command with the -r option, ' .
+                 'or run this as root'));
+    } else {
+        error(_g("gain-root-commmand '%s' not found"), $rootcommand[0]);
+    }
+}
+
+if ($check_command and not find_command($check_command)) {
+    error(_g("check-commmand '%s' not found"), $check_command);
+}
+
+if ($signcommand) {
+    if (!find_command($signcommand)) {
+        error(_g("sign-commmand '%s' not found"), $signcommand);
+    }
+} elsif (($ENV{GNUPGHOME} && -e $ENV{GNUPGHOME}) ||
+         ($ENV{HOME} && -e "$ENV{HOME}/.gnupg")) {
+    if (find_command('gpg2')) {
+        $signcommand = 'gpg2';
+    } elsif (find_command('gpg')) {
+        $signcommand = 'gpg';
     }
 }
 
-my $build_opts = Dpkg::BuildOptions->new();
 if (defined $parallel) {
+    if ($parallel eq 'auto') {
+        # Most Unices.
+        $parallel = qx(getconf _NPROCESSORS_ONLN 2>/dev/null);
+        # Fallback for at least Irix.
+        $parallel = qx(getconf _NPROC_ONLN 2>/dev/null) if $?;
+        chomp $parallel;
+    }
     $parallel = $build_opts->get('parallel') if $build_opts->has('parallel');
-    $ENV{MAKEFLAGS} ||= '';
+    $ENV{MAKEFLAGS} //= '';
     $ENV{MAKEFLAGS} .= " -j$parallel";
     $build_opts->set('parallel', $parallel);
     $build_opts->export();
@@ -298,9 +391,13 @@ my $changelog = changelog_parse();
 
 my $pkg = mustsetvar($changelog->{source}, _g('source package'));
 my $version = mustsetvar($changelog->{version}, _g('source version'));
-my ($ok, $error) = version_check($version);
+my $v = Dpkg::Version->new($version);
+my ($ok, $error) = version_check($v);
 error($error) unless $ok;
 
+my $sversion = $v->as_string(omit_epoch => 1);
+my $uversion = $v->version();
+
 my $distribution = mustsetvar($changelog->{distribution}, _g('source distribution'));
 
 my $maintainer;
@@ -330,15 +427,8 @@ if (build_sourceonly) {
     $arch = mustsetvar($ENV{DEB_HOST_ARCH}, _g('host architecture'));
 }
 
-if (!defined $signcommand &&
-    (($ENV{GNUPGHOME} && -e $ENV{GNUPGHOME}) ||
-     ($ENV{HOME} && -e "$ENV{HOME}/.gnupg"))) {
-    if (find_command('gpg2')) {
-        $signcommand = 'gpg2';
-    } elsif (find_command('gpg')) {
-        $signcommand = 'gpg';
-    }
-}
+my $pv = "${pkg}_$sversion";
+my $pva = "${pkg}_${sversion}_$arch";
 
 if (not $signcommand) {
     $signsource = 0;
@@ -352,12 +442,15 @@ if (not $signcommand) {
     $signchanges = 0;
 }
 
+if ($signsource && build_binaryonly) {
+    $signsource = 0;
+}
+
+#
 # Preparation of environment stops here
+#
 
-(my $sversion = $version) =~ s/^\d+://;
-
-my $pv = "${pkg}_$sversion";
-my $pva = "${pkg}_${sversion}_$arch";
+run_hook('init', 1);
 
 if (not -x 'debian/rules') {
     warning(_g('debian/rules is not executable; fixing that'));
@@ -371,9 +464,11 @@ unless ($call_target) {
 }
 
 if ($checkbuilddep) {
-    if ($admindir) {
-	push @checkbuilddep_opts, "--admindir=$admindir";
-    }
+    my @checkbuilddep_opts;
+
+    push @checkbuilddep_opts, '-A' if ($include & BUILD_ARCH_DEP) == 0;
+    push @checkbuilddep_opts, '-B' if ($include & BUILD_ARCH_INDEP) == 0;
+    push @checkbuilddep_opts, "--admindir=$admindir" if $admindir;
 
     system('dpkg-checkbuilddeps', @checkbuilddep_opts);
     if (not WIFEXITED($?)) {
@@ -402,10 +497,15 @@ if ($call_target) {
     exit 0;
 }
 
+run_hook('preclean', ! $noclean);
+
 unless ($noclean) {
     withecho(@rootcommand, @debian_rules, 'clean');
 }
-unless (build_binaryonly) {
+
+run_hook('source', $include & BUILD_SOURCE);
+
+if ($include & BUILD_SOURCE) {
     warning(_g('building a source package without cleaning up as you asked; ' .
                'it might contain undesired files')) if $noclean;
     chdir('..') or syserr('chdir ..');
@@ -413,6 +513,8 @@ unless (build_binaryonly) {
     chdir($dir) or syserr("chdir $dir");
 }
 
+run_hook('build', $include & BUILD_BINARY);
+
 if ($buildtarget ne 'build' and scalar(@debian_rules) == 1) {
     # Verify that build-{arch,indep} are supported. If not, fallback to build.
     # This is a temporary measure to not break too many packages on a flag day.
@@ -431,97 +533,84 @@ if ($buildtarget ne 'build' and scalar(@debian_rules) == 1) {
     }
 }
 
-unless (build_sourceonly) {
+if ($include & BUILD_BINARY) {
     withecho(@debian_rules, $buildtarget);
+    run_hook('binary', 1);
     withecho(@rootcommand, @debian_rules, $binarytarget);
 }
-if ($usepause &&
-    ($signchanges || (!build_binaryonly && $signsource))) {
-    print _g("Press the return key to start signing process\n");
-    getc();
-}
 
-my $signerrors;
-unless (build_binaryonly) {
-    if ($signsource && signfile("$pv.dsc")) {
-	$signerrors = _g('failed to sign .dsc and .changes file');
-	$signchanges = 0;
-    }
-}
+run_hook('changes', 1);
 
-if (defined($maint)) { push @changes_opts, "-m$maint" }
-if (defined($changedby)) { push @changes_opts, "-e$changedby" }
-if (defined($since)) { push @changes_opts, "-v$since" }
-if (defined($desc)) { push @changes_opts, "-C$desc" }
+push @changes_opts, "-m$maint" if defined $maint;
+push @changes_opts, "-e$changedby" if defined $changedby;
+push @changes_opts, "-v$since" if defined $since;
+push @changes_opts, "-C$desc" if defined $desc;
 
 my $chg = "../$pva.changes";
+my $changes = Dpkg::Control->new(type => CTRL_FILE_CHANGES);
+
 print { *STDERR } " dpkg-genchanges @changes_opts >$chg\n";
+
 open my $changes_fh, '-|', 'dpkg-genchanges', @changes_opts
     or subprocerr('dpkg-genchanges');
-
-open my $out_fh, '>', $chg or syserr(_g('write changes file'));
-
-my $infiles = my $files = '';
-while ($_ = <$changes_fh>) {
-    print { $out_fh } $_ or syserr(_g('write changes file'));
-    chomp;
-
-    if (/^Files:/i) {
-	$infiles = 1;
-    } elsif ($infiles && /^\s+(.*)$/) {
-	$files .= " $1 ";
-    } elsif ($infiles && /^\S/) {
-	$infiles = 0;
-    }
-}
-
+$changes->parse($changes_fh, _g('parse changes file'));
+$changes->save($chg);
 close $changes_fh or subprocerr(_g('dpkg-genchanges'));
-close $out_fh or syserr(_g('write changes file'));
 
-my $srcmsg;
-sub fileomitted($) { return $files !~ /$_[0]/ }
-my $ext = compression_get_file_extension_regex();
-if (fileomitted '\.deb') {
-    # source only upload
-    if (fileomitted "\.diff\.$ext" and fileomitted "\.debian\.tar\.$ext") {
-	$srcmsg = _g('source only upload: Debian-native package');
-    } elsif (fileomitted "\.orig\.tar\.$ext") {
-	$srcmsg = _g('source only, diff-only upload (original source NOT included)');
-    } else {
-	$srcmsg = _g('source only upload (original source is included)');
-    }
-} else {
-    $srcmsg = _g('full upload (original source is included)');
-    if (fileomitted '\.dsc') {
-	$srcmsg = _g('binary only upload (no source included)');
-    } elsif (fileomitted "\.diff\.$ext" and fileomitted "\.debian\.tar\.$ext") {
-	$srcmsg = _g('full upload; Debian-native package (full source is included)');
-    } elsif (fileomitted "\.orig\.tar\.$ext") {
-	$srcmsg = _g('binary and diff upload (original source NOT included)');
-    } else {
-	$srcmsg = _g('full upload (original source is included)');
-    }
-}
-
-if ($signchanges && signfile("$pva.changes")) {
-    $signerrors = _g('failed to sign .changes file');
-}
+run_hook('postclean', $cleansource);
 
 if ($cleansource) {
     withecho(@rootcommand, @debian_rules, 'clean');
 }
+
 chdir('..') or syserr('chdir ..');
 withecho('dpkg-source', @source_opts, '--after-build', $dir);
 chdir($dir) or syserr("chdir $dir");
 
+printf "$Dpkg::PROGNAME: %s\n", describe_build($changes->{'Files'});
+
+run_hook('check', $check_command);
+
+if ($check_command) {
+    withecho($check_command, @check_opts, $chg);
+}
+
+if ($signpause && ($signchanges || $signsource)) {
+    print _g("Press the return key to start signing process\n");
+    getc();
+}
+
+run_hook('sign', $signsource || $signchanges);
+
+if ($signsource) {
+    if (signfile("$pv.dsc")) {
+        error(_g('failed to sign .dsc and .changes file'));
+    }
+
+    # Recompute the checksums as the .dsc has changed now.
+    my $checksums = Dpkg::Checksums->new();
+    $checksums->add_from_control($changes);
+    $checksums->add_from_file("../$pv.dsc", update => 1, key => "$pv.dsc");
+    $checksums->export_to_control($changes);
+    delete $changes->{'Checksums-Md5'};
+
+    my $md5sum_regex = checksums_get_property('md5', 'regex');
+    my $dsc_md5sum = $checksums->get_checksum("$pv.dsc", 'md5');
+    my $dsc_size = $checksums->get_size("$pv.dsc");
+    my $dsc_files_regex = qr/$md5sum_regex\s+\d+\s+(\S+\s+\S+\s+\Q$pv\E\.dsc)/;
+    $changes->{'Files'} =~ s/^$dsc_files_regex$/$dsc_md5sum $dsc_size $1/m;
+
+    $changes->save($chg);
+}
+if ($signchanges && signfile("$pva.changes")) {
+    error(_g('failed to sign .changes file'));
+}
+
 if (not $signreleased) {
     warning(_g('not signing UNRELEASED build; use --force-sign to override'));
 }
-print "$Dpkg::PROGNAME: $srcmsg\n";
-if ($signerrors) {
-    warning($signerrors);
-    exit 1;
-}
+
+run_hook('done', 1);
 
 sub mustsetvar {
     my ($var, $text) = @_;
@@ -539,6 +628,39 @@ sub withecho {
 	and subprocerr("@_");
 }
 
+sub run_hook {
+    my ($name, $enabled) = @_;
+    my $cmd = $hook{$name};
+
+    return if not $cmd;
+
+    print { *STDERR } "$Dpkg::PROGNAME: running hook $name\n";
+
+    my %hook_vars = (
+        '%' => '%',
+        'a' => $enabled ? 1 : 0,
+        'p' => $pkg,
+        'v' => $version,
+        's' => $sversion,
+        'u' => $uversion,
+    );
+
+    my $subst_hook_var = sub {
+        my ($var) = @_;
+
+        if (exists $hook_vars{$var}) {
+            return $hook_vars{$var};
+        } else {
+            warning(_g('unknown %% substitution in hook: %%%s'), $var);
+            return "\%$var";
+        }
+    };
+
+    $cmd =~ s/\%(.)/&$subst_hook_var($1)/eg;
+
+    withecho($cmd);
+}
+
 sub signfile {
     my ($file) = @_;
     print { *STDERR } " signfile $file\n";
@@ -559,3 +681,35 @@ sub signfile {
     print "\n";
     return $status
 }
+
+sub fileomitted {
+    my ($files, $regex) = @_;
+
+    return $files !~ /$regex/
+}
+
+sub describe_build {
+    my ($files) = @_;
+    my $ext = compression_get_file_extension_regex();
+
+    if (fileomitted($files, qr/\.deb/)) {
+        # source-only upload
+        if (fileomitted($files, qr/\.diff\.$ext/) and
+            fileomitted($files, qr/\.debian\.tar\.$ext/)) {
+            return _g('source-only upload: Debian-native package');
+        } elsif (fileomitted($files, qr/\.orig\.tar\.$ext/)) {
+            return _g('source-only, diff-only upload (original source NOT included)');
+        } else {
+            return _g('source-only upload (original source is included)');
+        }
+    } elsif (fileomitted($files, qr/\.dsc/)) {
+        return _g('binary-only upload (no source included)');
+    } elsif (fileomitted($files, qr/\.diff\.$ext/) and
+             fileomitted($files, qr/\.debian\.tar\.$ext/)) {
+        return _g('full upload; Debian-native package (full source is included)');
+    } elsif (fileomitted($files, qr/\.orig\.tar\.$ext/)) {
+        return _g('binary and diff upload (original source NOT included)');
+    } else {
+        return _g('full upload (original source is included)');
+    }
+}
