@@ -95,7 +95,7 @@ foreach (@ARGV) {
 	$stdout = 1;
     } elsif (m/^-O(.+)$/) {
 	$varlistfile = $1;
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-(?:\?|-help)$/) {
 	usage(); exit(0);
     } elsif (m/^--version$/) {
 	version(); exit(0);
@@ -244,10 +244,8 @@ foreach my $file (keys %exec) {
             if (defined($symfile_path)) {
                 # Load symbol information
                 print "Using symbols file $symfile_path for $soname\n" if $debug;
-                unless (exists $symfile_cache{$symfile_path}) {
-                    $symfile_cache{$symfile_path} =
-                        Dpkg::Shlibs::SymbolFile->new(file => $symfile_path);
-                }
+                $symfile_cache{$symfile_path} //=
+                   Dpkg::Shlibs::SymbolFile->new(file => $symfile_path);
                 $symfile->merge_object_from_symfile($symfile_cache{$symfile_path}, $soname);
             }
 	    if (defined($symfile_path) && $symfile->has_object($soname)) {
@@ -267,9 +265,7 @@ foreach my $file (keys %exec) {
 	    } else {
 		# No symbol file found, fall back to standard shlibs
                 print "Using shlibs+objdump for $soname (file $lib)\n" if $debug;
-                unless (exists $objdump_cache{$lib}) {
-                    $objdump_cache{$lib} = Dpkg::Shlibs::Objdump::Object->new($lib);
-                }
+                $objdump_cache{$lib} //= Dpkg::Shlibs::Objdump::Object->new($lib);
                 my $libobj = $objdump_cache{$lib};
                 my $id = $dumplibs_wo_symfile->add_object($libobj);
 		if (($id ne $soname) and ($id ne $lib)) {
@@ -295,7 +291,9 @@ foreach my $file (keys %exec) {
 		    # 3/ when we have been asked to do so
 		    $ignore++ if $ignore_missing_info;
 		    error(_g('no dependency information found for %s ' .
-		             '(used by %s)'), $lib, $file)
+		             "(used by %s)\n" .
+		             'Hint: check if the library actually comes ' .
+		             'from a package.'), $lib, $file)
 		        unless $ignore;
 		}
 	    }
@@ -308,7 +306,7 @@ foreach my $file (keys %exec) {
     foreach (@sonames) {
         # Initialize statistics
         $soname_used{$_} = 0;
-        $global_soname_used{$_} = 0 unless exists $global_soname_used{$_};
+        $global_soname_used{$_} //= 0;
         if (exists $global_soname_needed{$_}) {
             push @{$global_soname_needed{$_}}, $file;
         } else {
@@ -538,7 +536,7 @@ foreach my $field (reverse @depfields) {
 # Replace old file by new one
 if (!$stdout) {
     close($fh) or syserr(_g('cannot close %s'), "$varlistfile.new");
-    rename("$varlistfile.new",$varlistfile)
+    rename "$varlistfile.new", $varlistfile
         or syserr(_g("install new varlist file \`%s'"), $varlistfile);
 }
 
@@ -638,7 +636,7 @@ sub add_shlibs_dep {
     my ($soname, $pkg, $libfile) = @_;
     my @shlibs = ($shlibslocal, $shlibsoverride);
     if ($pkg eq '') {
-	# If the file is not packaged, try to find out the shlibs file in
+	# If the file is not packaged, try to find out the shlibs file in
 	# the package being built where the lib has been found
 	my $pkg_root = guess_pkg_root_dir($libfile);
 	if (defined $pkg_root) {
@@ -701,7 +699,7 @@ sub extract_from_shlibs {
 	            $shlibfile, $., $_);
 	    next;
 	}
-	my $depread = defined($4) ? $4 : '';
+	my $depread = $4 // '';
 	if (($libname eq $2) && ($libversion eq $3)) {
 	    # Define dep and end here if the package type explicitly
 	    # matches. Otherwise if the packagetype is not specified, use
@@ -725,7 +723,7 @@ sub find_symbols_file {
     my ($pkg, $soname, $libfile) = @_;
     my @files;
     if ($pkg eq '') {
-	# If the file is not packaged, try to find out the symbols file in
+	# If the file is not packaged, try to find out the symbols file in
 	# the package being built where the lib has been found
 	my $pkg_root = guess_pkg_root_dir($libfile);
 	if (defined $pkg_root) {
@@ -860,7 +858,8 @@ sub find_packages {
 	    print { *STDERR } " $_\n"
 		or syserr(_g('write diversion info to stderr'));
 	} elsif (m/^([-a-z0-9+.:, ]+): (\/.*)$/) {
-	    $cached_pkgmatch{$2} = $pkgmatch->{$2} = [ split(/, /, $1) ];
+	    my ($pkgs, $path) = ($1, $2);
+	    $cached_pkgmatch{$path} = $pkgmatch->{$path} = [ split /, /, $pkgs ];
 	} else {
 	    warning(_g("unknown output from dpkg --search: '%s'"), $_);
 	}
